/******************************************************************************

  Copyright (c) 2005-2011 by Turku PET Centre

  File:        libtpcimgiov.c
  Description: This file contains function for printing the date and time when
               this library is compiled.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU Lesser General Public License for more details:
  http://www.gnu.org/copyleft/lesser.html

  You should have received a copy of the GNU Lesser General Public License
  along with this library/program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 

  Turku PET Centre, Turku, Finland, http://www.turkupetcentre.fi
  
  Modification history:
  2004-11-07 Vesa Oikonen
  2005-01-20 Calle Laakkonen
  2005-04-26 Jarkko Johansson
  Functions libtpcimgio_print_readme(), libtpcimgio_print_history(),
  libtpcimgio_print_dreadme() and libtpcimgio_print_dhistory()
  are used for getting the Readme and History messages for the libtpcimgio,
  with and without Doxygen style tags. Remember to update the
  functions to get correct Readme and History messages!

  Function libtpcimgio_print_build(), is used for getting the build
  information.

  2005-08-01 1.1.1 CL/JJ
  Changed image loading/writing functions fname to const,
  and extern int VOL_TEST to int VOL_TEST.

  2005-10-10 1.1.2 CL
  See function libtpcimgio_print_history().

  2005-11-21 1.1.3 VO
  See halflife.c.

  2005-12-12 1.1.4 VO
  See imgfile.c.

  2006-10-31 1.1.5 VO
  Created imgunit.c; moved a few functions into it.
  Fixed a timezone bug in imgfile.c.

  2006-12-28 1.1.6 VO
  Changes in halflife.c.
  
  2007-02-02 1.2.0 VO
  Changes in IMG structure.
  Halflife.c, *.h, and related functions moved into libtpcmisc 1.2.0.
  Not more dependent from libtpcimgp: min and max search moved to this library.
  Lots of changes in imgunit.c.

  2007-02-11 1.2.1 VO
  Changes and additions regarding statmsg processing and storing.
  Deleted imgmsg.h.
  Added errstatus in IMG struct.
  
  2007-02-27 1.2.2 VO
  Bug corrections.
  ECAT7 related functions separated from imgfile.c to img_e7.c.
  Added functions for Analyze, Ecat 6.3 and Ecat 7 formats.
  
  2007-03-12 1.2.3 VO
  ECAT 6.3 header print functions changed.
  Added functions for ECAT matrix lists.
  Added IMG error messages.
  
  2007-03-21 1.2.4 VO
  Changes to make easier to save ECAT7 2D data.
  ECAT7 main header scan start time printed correctly when compiled with Mingw.
  Bug correction in ecat7ReadImageheader() in extra data contents.

  2007-03-25 1.2.5 VO

  2007-04-03 1.3.0 VO
  Added support for ECAT7 polar map.

  2007-07-17 Harri Merisaari
  Modified for optional ANSI compatibility

  2007-09-10 1.3.2 VO
  Bug corrections in img.c and img_e7.c.

  2007-09-10 1.3.3 VO
  Return value of localtime() is checked.

  2007-09-11 1.3.4 VO
  Bug correction in analyze.c.

  2007-09-12 1.3.5 VO
  Bug correction in img_e7.c.

  2008-07-07 1.3.6 VO
  Changes in imgdecay.c and img_ana.c.

  2008-07-11 1.3.7 VO
  Addition in weight.c and sif.h.

  2008-07-14 1.3.8 VO
  See history below.

  From this on, see history below.

******************************************************************************/
#include <stdio.h>
#include "libtpcimgiov.h"
/*****************************************************************************/
/** Library name. */
#define LIB_NAME "libtpcimgio"

/** Library version. */
#define LIB_VERSION "1.5.10"

/** Copyright. */
#define COPYRIGHT "(c) 2005-2011 by Turku PET Centre"

/*****************************************************************************/
/*!
 * Print the library readme to specified FILE pointer, use stdout for screen.
 *
 * @param fp file pointer
 */
void libtpcimgio_print_readme(FILE *fp){
  fputs("\n"
	LIB_NAME" "LIB_VERSION" "COPYRIGHT"\n"
	"\n"
	"Build "__DATE__" "__TIME__"\n"
	"\n"
	"\n"
	"PET Image IO library\n"
	"-------------------------------------------------------------------------------\n"
	"\n"
	"2007-03-21\n"
	"Turku PET Centre\n"
	"\n",fp);
  fputs("Libtpcimgio is distributed under the GNU Lesser General Public Licence\n"
	"http://www.gnu.org/licenses/lgpl.html\n"
	"More information on open source licences: http://www.opensource.org/\n"
	"\n"
	"Description\n"
	"-------------------------------------------------------------------------------\n"
	"\n"
	"The libtpcimgio library is a collection of commonly used C files in Turku PET Centre\n"
	"for image file input and output procedures. Libtpcimgio library supports Analyze 7.5,\n"
	"Ecat 6.x, Ecat 7.x and partly interfile formats.\n"
	"\n",fp);
  fputs("List of files\n"
	"-------------------------------------------------------------------------------\n"
	"\n"
	"include/     - Includes all the header files for the .c files\n"
	"\n"
	"Makefile     - The makefile\n"
	"\n"
	"** .c files **\n"
	"\n"
	"analyze.c     - reading and writing Analyze 7.5 image files.\n"
	"\n"
	"ecat63ml.c    - Procedures for ECAT 6.3 matrix list.\n"
	"\n",fp);
  fputs("ecat63p.c     - Printing ECAT 6.3 headers.\n"
	"\n"
	"ecat63r.c     - Reading ECAT 6.3 files.\n"
	"\n"
	"ecat63w.c     - Writing ECAT 6.3 files.\n"
	"\n"
	"ecat7ml.c     - Procedures for ECAT 7.x matrix list.\n"
	"\n"
	"ecat7p.c      - Printing ECAT 7.x headers.\n"
	"\n"
	"ecat7h.c      - Edit ECAT 7.x headers.\n"
	"\n"
	"ecat7r.c      - Reading ECAT 7.x files.\n"
	"\n",fp);
  fputs("ecat7w.c      - Writing ECAT 7.x files.\n"
	"\n"
	"img.c         - Procedures for 4D image data (common to all image file formats).\n"
	"\n"
	"img_ana.c     - Subprocedures for reading/writing Analyze 7.5 files to/from IMG.\n"
	"\n"
	"img_e63.c     - Subprocedures for reading/writing ECAT 6.3 files to/from IMG.\n"
	"\n"
	"img_e7.c      - Subprocedures for reading/writing ECAT 7 files to/from IMG.\n"
	"\n"
	"imgdecay.c    - Decay correction, isotope codes, and half-lives.\n"
	"\n"
	"imgfile.c     - Main procedures for IMG read/write.\n"
	"\n",fp);
  fputs("imgmax.c      - Procedures for searching IMG min and max value.\n"
	"\n"
	"imgunit.c     - Procedures for working with IMG calibration unit.\n"
	"\n"
	"interfile.c   - Reading Interfile headers.\n"
	"\n"
	"micropet.c    - Reading Inveon images.\n"
	"\n"
	"sif.c         - Routines for Scan Information Files (SIF).\n"
	"\n"
	"sifio.c       - Functions for reading and writing SIF format files.\n"
	"\n"
	"vol.c         - Procedures for 3D image volume data (no time information).\n"
	"\n"
	"weight.c      - Functions for setting weight factors based on SIF.\n"
	"\n",fp);
  fputs("libtpcimgio.c - Program for printing Readme and History information, and for\n"
	"		 testing the library functions.\n"
	"\n"
	"libtpcimgiov.c- Version information for this library.\n"
	"\n"
	"\n"
	"Using the library\n"
	"-------------------------------------------------------------------------------\n"
	"\n"
	"The program that uses the library should include the .h file it needs.\n"
	"Example:\n"
	"    \\#include <img.h>\n"
	"\n"
	"libtpcimgio depends on libtpcmisc.\n"
	"Example:\n"
	"	gcc myprog.c -ltpcimgio -ltpcmisc\n"
	"\n",fp);
  fflush(fp);
}

/*!
 * Print the library History to specified FILE pointer,
 * use stdout for screen.
 * 
 * @param fp file pointer
 */
void libtpcimgio_print_history(FILE *fp) {
  fputs("2005-01-04 1.0.0 Vesa Oikonen, Calle Laakkonen, Riku Kln, Jarkko Johansson\n"
	"	Former libpet, the common PET C library, has been divided up in smaller\n"
	"	sublibraries that each handle a specific task. This library includes functions\n"
	"	utilised in image file input and output procedures. \n"
	"\n"
	"2005-03-03 1.0.0 Jarkko Johansson \n"
	"	imgproc.[c/h] moved to libtpcimgp library\n"
	"\n"
	"2005-04-05 1.0.1 (krs) Roman Krais\n"
	"       interfile.c added (for reading interfile headers)\n"
	"\n"
	"2005-04-25 1.1.0 Jarkko Johansson\n",fp);
  fputs("	History and Readme files included in new print functions:\n"
	"	libtpcimgio_print_readme(FILE *fp);\n"
	"	libtpcimgio_print_history(FILE *fp);\n"
	"	libtpcimgio_print_dreadme(FILE *fp);\n"
        "	libtpcimgio_print_dhistory(FILE *fp);\n"
	"	libtpcimgio_print_build(FILE *fp);\n"
	"	in file libtpcimgio.c. Update the the Readme and History\n"
	"	there.\n"
	"\n"
	"	Compiled in Windows with MinGW.\n"
	"\n"
	"2005-05-04 1.1.0 JJ\n"
	"       Minor changes in file sifio.c.\n"
	"\n"
	"2005-08-01 1.1.1 CL/JJ\n",fp);
  fputs("       Changed image loading/writing functions fname to const,\n"
	"       and extern int VOL_TEST to int VOL_TEST.\n"
	"\n"
	"2005-10-10 1.1.2 CL\n"
	"       imgWriteAnalyze() now writes only 1 frame at a time,\n"
	"       conserving memory.\n"
	"\n"
	"2005-11-21 1.1.3 VO\n"
	"       lambda2factor() now calculates correctly when removing\n"
	"       decay correction considering frame lengths.\n"
	"\n"
	"2005-12-12 1.1.4 VO\n"
	"       imgReadAnalyze() sets img.isotopeHalflife,\n"
	"       if isotope is found in SIF.\n"
	"\n",fp);
  fputs("2006-10-31 1.1.5 VO\n"
	"       Added function imgUnitId().\n"
	"       Fixed a timezone bug with ECAT7 files.\n"
	"\n"
	"2006-12-28 1.1.6 VO\n"
	"       Changes in identification of isotope codes.\n"
	"\n"
	"2007-02-02 1.2.0 VO\n"
	"       Additions in IMG structure, and subsequent changes in i/o functions.\n"
	"       Isotope and halflife related functions are relocated into libtpcmisc.\n"
	"       This library is now independent of libtpcimgp.\n",fp);
  fputs("       IMG min and max search functions are relocated here.\n"
        "       User_process_code is read/written with ECAT7 and ECAT6.3 files.\n"
        "       Patient_id is read/written with ECAT7 and ECAT6.3 files.\n"
        "       Study_description is read/written with ECAT7 and ECAT6.3 files.\n"
        "       X, y and z resolution is read/written with ECAT7 images.\n"
        "       Prompts and randoms (delayed) are read/written with ECAT7 and\n"
	"       ECAT6.3 sinograms and read from SIF with Analyze 7.5.\n",fp);
  fputs("       Changes in identifying and processing calibration units.\n"
	"\n"
	"2007-02-11 1.2.1 VO\n"
	"       Addition in IMG structure.\n"
	"       Deleted imgmsg.h. Set IMG.statmsg using imgSetStatus().\n"
	"\n"
	"2007-02-27 1.2.2 VO\n"
	"       Bug corrections.\n"
	"\n"
	"2007-03-13 1.2.3 VO\n"
        "       Added ECAT matrix list functions and error messages.\n"
	"       ECAT 6.3 header print format changed.\n"
	"\n",fp);
  fputs("2007-03-21 1.2.4 VO\n"
	"       Changes to make easier to write ECAT7 2D files.\n"
	"\n"
	"2007-03-25 1.2.5 VO\n"
	"       Addition of functions for reading and writing ECAT and Analyze\n"
        "       format one frame at a time.\n"
	"\n"
	"2007-04-03 1.3.0 VO\n"
	"       Added support for ECAT7 polar maps.\n"
        "       Information for polar map added to IMG struct.\n"
        "\n"
        "2007-07-17 1.3.1 Harri Merisaari\n"
        "       Modified for optional ANSI compatibility.\n"
        "\n"
        "2007-09-10 1.3.2 VO\n"
        "       Bug correction in setting error messages.\n"
        "       Bug correction in reading some too long header text fields.\n"
        "\n"
        "2007-09-10 1.3.3 VO\n"
        "       Return value of localtime() is always checked.\n"
	"\n",fp);
  fputs("2007-09-11 1.3.4 VO\n"
	"       Corrected a bug affecting versions 1.3.1 -1.3.3:\n"
        "       Analyze and ECAT files could have been mixed if in same folder.\n"
        "\n"
        "2007-09-12 1.3.5 VO\n"
	"       Corrected a bug affecting frame-by-frame reading of\n"
        "       ECAT7 2D image files.\n"
        "\n"
        "2008-07-07 1.3.6 VO\n"
	"       Image decay correction functions return error if frame times\n"
        "       are missing.\n"
        "       Assumes that Analyze 7.5 images are corrected for decay when\n"
        "       that information is not found in header file.\n"
        "\n"
        "2008-07-11 1.3.7 VO\n"
	"       Added function sifModerate().\n"
        "\n"
        "2008-07-14 1.3.8 VO\n"
	"       Added function imgAllocateWithHeader().\n"
        "\n"
        "2008-07-24 1.3.9 VO\n"
	"       Added functions ecat7EditMHeader(), ecat7EditSHeader(),\n"
        "       ecat7EditVHeader(), imgSetUnit(), and ecat7PrintSubheader().\n"
        "\n"
        "2008-09-02 1.4.0 VO\n"
	"       Default ECAT backup extension changed to .bak.\n"
        "\n"
        "2008-10-03 1.4.1 VO\n"
	"       Accepts certain Analyze formats with 32 bit pixel values.\n"
        "\n"
        "2008-10-09 1.4.2 VO\n"
	"       Accepts more Analyze formats with 32 bit pixel values (PVElab).\n"
	"\n",fp);
  fputs("2008-11-06 1.4.3 VO\n"
	"       Bug correction in reading ECAT 6.3 files.\n"
        "\n"
        "2009-02-17 1.5.0 VO\n"
	"       Added definitions for Siemens Inveon in img.h.\n"
	"       Added micropet.c and .h for reading Siemens Inveon images.\n"
        "\n"
        "2009-02-25 1.5.1 VO\n"
	"       Changes in micropet.c and .h to allow reading CT images.\n"
        "\n"
        "2009-04-06 1.5.2 VO\n"
	"       Changes in sif.c to allow SIF without prompts and randoms.\n"
        "\n"
        "2009-09-28 1.5.3 VO\n"
	"       Change in micropet.c reading pixel z size.\n"
        "\n"
        "2009-12-01 1.5.4 VO\n"
	"       Added function imgSmoothMax().\n"
        "\n"
        "2009-12-10 1.5.5 VO\n"
	"       Tiny update in img_ana.c.\n"
        "\n"
        "2010-02-12 1.5.6 VO\n"
	"       Time zone is corrected in imgSetEcat7MHeader().\n"
        "\n"
        "2010-05-11 1.5.7 VO\n"
	"       imgUnitFromEcat7() tries more to identify units.\n"
        "\n"
        "2010-08-19 1.5.8 VO\n"
	"       ECAT7 main header field patient_birth_date is read correctly\n"
        "       whether is is in documented YYYYMMDD format or as seconds from\n"
        "       start of years 1970.\n" 
        "\n"
        "2011-01-11 1.5.9 VO\n"
	"       Bug fixes in reading micropet images.\n"
        "\n"
        "2011-02-04 1.5.10 VO\n"
	"       Label unitless was previously not written in main header\n"
        "       when IMG data is written in ECAT7 format.\n"
	"\n",fp);
  fflush(fp);  
}

/*!
 * Print the library Readme to specified FILE pointer with Doxygen tags,
 * use stdout for screen.
 *
 * @param fp file pointer where to print
 */
void libtpcimgio_print_dreadme(FILE *fp) {
  fprintf(fp,"/**\n\\mainpage %s %s %s\n\\section readme Readme\n<PRE>\n",LIB_NAME,LIB_VERSION,COPYRIGHT);
  libtpcimgio_print_readme(fp);
  fprintf(fp,"\n</PRE> \n */\n");
}

/*!
 * Print the library History to specified FILE pointer with Doxygen tags,
 * use stdout for screen.
 *
 * @param fp file pointer where to print
 */
void libtpcimgio_print_dhistory(FILE *fp) {
  fprintf(fp,"/**\n\\page History\n\\section history History\n<PRE>\n");
  libtpcimgio_print_history(fp);
  fprintf(fp,"\n</PRE> \n */\n");
}

/*!
 * Print the build info to specified FILE pointer, use stdout for screen.
 *
 * @param fp file pointer where to print
 */
void libtpcimgio_print_build(FILE *fp) {
  fprintf(fp,"\n %s %s %s\n",LIB_NAME,LIB_VERSION,COPYRIGHT);
  fprintf(fp,"\n Build %s %s\n",__DATE__,__TIME__);
#ifdef __STRICT_ANSI__
  fprintf(fp,"\nANSI compatible build\n");
#endif
}

/*!
 * Print the compilation date and time to specified FILE pointer
 *
 * @param fp file pointer where to print
 */
void libtpcimgio_printdate(FILE *fp) {
  libtpcimgio_print_build(fp);
}

/*****************************************************************************/

/*****************************************************************************/

